@@ -6,6 +6,7 @@ from __future__ import unicode_literals
 __all__ = ('TreeForeignKey', 'TreeOneToOneField', 'TreeManyToManyField')
 
 from django.db import models
+from django.conf import settings
 from mptt.forms import TreeNodeChoiceField, TreeNodeMultipleChoiceField
 
 
@@ -38,10 +39,8 @@ class TreeManyToManyField(models.ManyToManyField):
         return super(TreeManyToManyField, self).formfield(**kwargs)
 
 # South integration
-try:  # pragma: no cover
+if 'south' in settings.INSTALLED_APPS:  # pragma: no cover
     from south.modelsinspector import add_introspection_rules
     add_introspection_rules([], ["^mptt\.fields\.TreeForeignKey"])
     add_introspection_rules([], ["^mptt\.fields\.TreeOneToOneField"])
     add_introspection_rules([], ["^mptt\.fields\.TreeManyToManyField"])
-except ImportError:
-    pass
