@@ -3,23 +3,16 @@ Template tags for working with lists of model instances which represent
 trees.
 """
 from __future__ import unicode_literals
-import warnings
 
 from django import template
-try:
-    from django.apps import apps
-    get_model = apps.get_model
-except ImportError:  # pragma: no cover (Django 1.6 compatibility)
-    from django.db.models import get_model
+from django.apps import apps
 from django.db.models.fields import FieldDoesNotExist
-try:
-    from django.utils.encoding import force_text
-except ImportError:  # pragma: no cover (Django 1.4 compatibility)
-    from django.utils.encoding import force_unicode as force_text
+from django.utils.encoding import force_text
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
 
-from mptt.utils import tree_item_iterator, drilldown_tree_for_node
+from mptt.utils import (drilldown_tree_for_node, get_cached_trees,
+                        tree_item_iterator)
 
 register = template.Library()
 
@@ -32,7 +25,7 @@ class FullTreeForModelNode(template.Node):
         self.context_var = context_var
 
     def render(self, context):
-        cls = get_model(*self.model.split('.'))
+        cls = apps.get_model(*self.model.split('.'))
         if cls is None:
             raise template.TemplateSyntaxError(
                 _('full_tree_for_model tag was given an invalid model: %s') % self.model
@@ -56,7 +49,7 @@ class DrilldownTreeForNodeNode(template.Node):
 
         if self.foreign_key is not None:
             app_label, model_name, fk_attr = self.foreign_key.split('.')
-            cls = get_model(app_label, model_name)
+            cls = apps.get_model(app_label, model_name)
             if cls is None:
                 raise template.TemplateSyntaxError(
                     _('drilldown_tree_for_node tag was given an invalid model: %s') %
@@ -239,83 +232,10 @@ def tree_path(items, separator=' :: '):
 @register.filter
 def cache_tree_children(queryset):
     """
-    Takes a list/queryset of model objects in MPTT left (depth-first) order,
-    caches the children on each node, as well as the parent of each child node,
-    allowing up and down traversal through the tree without the need for
-    further queries. This makes it possible to have a recursively included
-    template without worrying about database queries.
-
-    Returns a list of top-level nodes. If a single tree was provided in its
-    entirety, the list will of course consist of just the tree's root node.
+    Alias to `mptt.utils.get_cached_trees`.
     """
 
-    current_path = []
-    top_nodes = []
-
-    # If ``queryset`` is QuerySet-like, set ordering to depth-first
-    if hasattr(queryset, 'order_by'):
-        mptt_opts = queryset.model._mptt_meta
-        tree_id_attr = mptt_opts.tree_id_attr
-        left_attr = mptt_opts.left_attr
-        if tuple(queryset.query.order_by) != (tree_id_attr, left_attr):
-            warnings.warn(
-                (
-                    "cache_tree_children() was passed a queryset with the wrong ordering: %r.\n"
-                    "This will cause an error in mptt 0.8."
-                ) % (queryset.query.order_by,),
-                UserWarning,
-            )
-            queryset = queryset.order_by(tree_id_attr, left_attr)
-
-    if queryset:
-        # Get the model's parent-attribute name
-        parent_attr = queryset[0]._mptt_meta.parent_attr
-        root_level = None
-        for obj in queryset:
-            # Get the current mptt node level
-            node_level = obj.get_level()
-
-            if root_level is None:
-                # First iteration, so set the root level to the top node level
-                root_level = node_level
-
-            if node_level < root_level:
-                # ``queryset`` was a list or other iterable (unable to order),
-                # and was provided in an order other than depth-first
-                raise ValueError(
-                    _('Node %s not in depth-first order') % (type(queryset),)
-                )
-
-            # Set up the attribute on the node that will store cached children,
-            # which is used by ``MPTTModel.get_children``
-            obj._cached_children = []
-
-            # Remove nodes not in the current branch
-            while len(current_path) > node_level - root_level:
-                current_path.pop(-1)
-
-            if node_level == root_level:
-                # Add the root to the list of top nodes, which will be returned
-                top_nodes.append(obj)
-            else:
-                # Cache the parent on the current node, and attach the current
-                # node to the parent's list of children
-                _parent = current_path[-1]
-                setattr(obj, parent_attr, _parent)
-                _parent._cached_children.append(obj)
-
-                if root_level == 0:
-                    # get_ancestors() can use .parent.parent.parent...
-                    setattr(obj, '_mptt_use_cached_ancestors', True)
-
-            # Add the current node to end of the current path - the last node
-            # in the current path is the parent for the next iteration, unless
-            # the next iteration is higher up the tree (a new branch), in which
-            # case the paths below it (e.g., this one) will be removed from the
-            # current path during the next iteration
-            current_path.append(obj)
-
-    return top_nodes
+    return get_cached_trees(queryset)
 
 
 class RecurseTreeNode(template.Node):
