@@ -1,27 +1,15 @@
 from __future__ import unicode_literals
-import uuid
 from django.db import models
-from django.db.models.signals import pre_save
 from django.utils.encoding import python_2_unicode_compatible
-from django import VERSION as DJANGO_VERSION
-
 
 import mptt
 from mptt.fields import TreeForeignKey, TreeOneToOneField, TreeManyToManyField
 from mptt.models import MPTTModel
 from mptt.managers import TreeManager
-from django.db.models.query import QuerySet, EmptyQuerySet
+from django.db.models.query import QuerySet
 
 
 class CustomTreeQueryset(QuerySet):
-
-    def custom_method(self):
-        pass
-
-
-class CustomEmptyTreeQueryset(EmptyQuerySet):
-    """This is only used pre Django 1.6"""
-
     def custom_method(self):
         pass
 
@@ -35,20 +23,15 @@ class CustomTreeManager(TreeManager):
         return CustomTreeQueryset(model=self.model, using=self._db)
 
     def get_empty_query_set(self):
-        # Pre 1.6 django, we needed a custom inheritor of EmptyQuerySet
-        # to pass custom methods. However, 1.6 introduced that EmptyQuerySet
-        # cannot be instantiated but instead passes through the methods
-        # of the custom QuerySet.
-        # See: https://code.djangoproject.com/ticket/22817
-        if DJANGO_VERSION < (1, 6):
-            return CustomEmptyTreeQueryset(model=self.model)
         return self.get_queryset().none()
 
 
 @python_2_unicode_compatible
 class Category(MPTTModel):
     name = models.CharField(max_length=50)
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
     category_uuid = models.CharField(max_length=50, unique=True, null=True)
 
     def __str__(self):
@@ -63,8 +46,12 @@ class Category(MPTTModel):
 class Item(models.Model):
 
     name = models.CharField(max_length=100)
-    category_fk = models.ForeignKey('Category', to_field='category_uuid', null=True, related_name='items_by_fk')
-    category_pk = models.ForeignKey('Category', null=True, related_name='items_by_pk')
+    category_fk = models.ForeignKey(
+        'Category', to_field='category_uuid', null=True,
+        related_name='items_by_fk', on_delete=models.CASCADE)
+    category_pk = models.ForeignKey(
+        'Category', null=True, related_name='items_by_pk',
+        on_delete=models.CASCADE)
 
     def __str__(self):
         return self.name
@@ -73,14 +60,16 @@ class Item(models.Model):
 @python_2_unicode_compatible
 class Genre(MPTTModel):
     name = models.CharField(max_length=50, unique=True)
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
     def __str__(self):
         return self.name
 
 
 class Game(models.Model):
-    genre = TreeForeignKey(Genre)
+    genre = TreeForeignKey(Genre, on_delete=models.CASCADE)
     genres_m2m = models.ManyToManyField(Genre, related_name='games_m2m')
     name = models.CharField(max_length=50)
 
@@ -89,7 +78,9 @@ class Game(models.Model):
 
 
 class Insert(MPTTModel):
-    parent = models.ForeignKey('self', null=True, blank=True, related_name='children')
+    parent = models.ForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
 
 @python_2_unicode_compatible
@@ -97,7 +88,9 @@ class MultiOrder(MPTTModel):
     name = models.CharField(max_length=50)
     size = models.PositiveIntegerField()
     date = models.DateField()
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
     class MPTTMeta:
         order_insertion_by = ['name', 'size', '-date']
@@ -107,7 +100,9 @@ class MultiOrder(MPTTModel):
 
 
 class Node(MPTTModel):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
     # To check that you can set level_attr etc to an existing field.
     level = models.IntegerField()
 
@@ -121,7 +116,9 @@ class Node(MPTTModel):
 @python_2_unicode_compatible
 class OrderedInsertion(MPTTModel):
     name = models.CharField(max_length=50)
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
     class MPTTMeta:
         order_insertion_by = ['name']
@@ -131,11 +128,15 @@ class OrderedInsertion(MPTTModel):
 
 
 class Tree(MPTTModel):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
 
 class NewStyleMPTTMeta(MPTTModel):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
     class MPTTMeta(object):
         left_attr = 'testing'
@@ -144,11 +145,13 @@ class NewStyleMPTTMeta(MPTTModel):
 @python_2_unicode_compatible
 class Person(MPTTModel):
     name = models.CharField(max_length=50)
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
     # just testing it's actually possible to override the tree manager
     objects = CustomTreeManager()
-    
+
     # This line is set because of https://github.com/django-mptt/django-mptt/issues/369
     _default_manager = objects
 
@@ -166,15 +169,16 @@ class CustomPKName(MPTTModel):
     name = models.CharField(max_length=50)
     parent = TreeForeignKey(
         'self', null=True, blank=True,
-        related_name='children', db_column="my_cusom_parent")
+        related_name='children', db_column="my_cusom_parent",
+        on_delete=models.CASCADE)
 
     def __str__(self):
         return self.name
 
 
 class ReferencingModel(models.Model):
-    fk = TreeForeignKey(Category, related_name='+')
-    one = TreeOneToOneField(Category, related_name='+')
+    fk = TreeForeignKey(Category, related_name='+', on_delete=models.CASCADE)
+    one = TreeOneToOneField(Category, related_name='+', on_delete=models.CASCADE)
     m2m = TreeManyToManyField(Category, related_name='+')
 
 
@@ -183,7 +187,9 @@ class ReferencingModel(models.Model):
 # 1. multi-table inheritance, with mptt fields on base class.
 
 class MultiTableInheritanceA1(MPTTModel):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
 
 class MultiTableInheritanceA2(MultiTableInheritanceA1):
@@ -197,13 +203,17 @@ class MultiTableInheritanceB1(MPTTModel):
 
 
 class MultiTableInheritanceB2(MultiTableInheritanceB1):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
 
 
 # 3. abstract models
 
 class AbstractModel(MPTTModel):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
     ghosts = models.CharField(max_length=50)
 
     class Meta:
@@ -244,8 +254,28 @@ class DoubleProxyModel(SingleProxyModel):
         proxy = True
 
 
+# 5. swappable models
+
+class SwappableModel(MPTTModel):
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
+
+    class Meta:
+        swappable = 'MPTT_SWAPPABLE_MODEL'
+
+
+class SwappedInModel(MPTTModel):
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
+    name = models.CharField(max_length=50)
+
+
 class AutoNowDateFieldModel(MPTTModel):
-    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
+    parent = TreeForeignKey(
+        'self', null=True, blank=True, related_name='children',
+        on_delete=models.CASCADE)
     now = models.DateTimeField(auto_now_add=True)
 
     class MPTTMeta:
@@ -256,5 +286,7 @@ class AutoNowDateFieldModel(MPTTModel):
 class Group(models.Model):
     name = models.CharField(max_length=100)
 
-TreeForeignKey(Group, blank=True, null=True).contribute_to_class(Group, 'parent')
+TreeForeignKey(
+    Group, blank=True, null=True, on_delete=models.CASCADE
+).contribute_to_class(Group, 'parent')
 mptt.register(Group, order_insertion_by=('name',))
